// NumEdit.cpp : implementation file
//

#include "stdafx.h"
#include <sstream>
#include "NumEdit.h"

const TCHAR CNumEdit::NEGATION_SYMBOL = _T('-');
const TCHAR CNumEdit::DECIMAL_SYMBOL = _T('.');
const COLORREF CNumEdit::NEGATIVE_COLOR = RGB(150, 50, 50);
const COLORREF CNumEdit::POSITIVE_COLOR = RGB(0, 0, 0);
const int CNumEdit::MAX_LENGTH = 9;

BEGIN_MESSAGE_MAP(CNumEdit, CEdit)
   ON_MESSAGE(WM_PASTE, OnPaste)
   ON_WM_CTLCOLOR_REFLECT()
   ON_WM_MOUSEWHEEL()
   ON_WM_CHAR()
   ON_WM_KILLFOCUS()
END_MESSAGE_MAP()

CNumEdit::CNumEdit()
{
   m_allowNegativeValues = false;
   m_isNegative = false;
}

BOOL CNumEdit::OnMouseWheel(
                        UINT /*nFlags*/,
                        short zDelta,
                        CPoint /*pt*/)
{
   IncreaseValue(zDelta / WHEEL_DELTA);
   return TRUE;
}

LRESULT CNumEdit::OnPaste(
                        WPARAM /*wParam*/,
                        LPARAM /*lParam*/)
{
   return 0L;
}

HBRUSH CNumEdit::CtlColor(
                        CDC* pDC,
                        UINT /*nCtlColor*/) 
{
   // Set text color
   pDC->SetTextColor(m_isNegative ? NEGATIVE_COLOR : POSITIVE_COLOR);

   // Return background brush.
   return ::GetSysColorBrush(COLOR_WINDOW);
}

BOOL CNumEdit::PreTranslateMessage(
                        MSG *pMSG)
{
	if ((pMSG->message == WM_KEYDOWN) || (pMSG->message == WM_KEYUP))
   {
      // Handle key up, increase value by 1
      if ((pMSG->message == WM_KEYDOWN) && (pMSG->wParam == VK_UP))
      {
         IncreaseValue(1);
         return TRUE;
      }
      // Handle key down, decrease value by 1
      else if ((pMSG->message == WM_KEYDOWN) && (pMSG->wParam == VK_DOWN))
      {
         DecreaseValue(1);
         return TRUE;
      }
   }

   return CEdit::PreTranslateMessage(pMSG);
}

void CNumEdit::OnKillFocus(CWnd* pNewWnd)
{
   CEdit::OnKillFocus(pNewWnd);

   // Get edit box text
   CString valueText;
   GetWindowText(valueText);

   // Validate if new text is different from last set text
   if (m_lastText == valueText)
   {
      return;
   }

   double value;

   if (valueText.IsEmpty() || valueText == NEGATION_SYMBOL)
   {
      // If new text is empty or contain only negation symbol then set value to 0
      value = 0;
      m_isNegative = false;
   }
   else
   {
      // Remove first character if it is a negation symbol
      m_isNegative = (valueText[0] == NEGATION_SYMBOL);
      if (m_isNegative)
      {
         valueText.Delete(0);
      }

      // Convert to a number and multiply by -1 if negative
      value = (m_isNegative ? -1 : 1) * _tstof(valueText);
   }

   // Set validated text into edit control
   SetEditBoxValue(value);
}

void CNumEdit::OnChar(
                        UINT nChar,
                        UINT nRepCnt,
                        UINT nFlags) 
{
   if (nChar == DECIMAL_SYMBOL)
   {
      CString valueText;
      GetWindowText(valueText);
      if (valueText.Find(DECIMAL_SYMBOL) < 0)
      {
         // Accept only one '.'
         CEdit::OnChar(nChar, nRepCnt, nFlags);
         return;
      }
   }
   else if (nChar == NEGATION_SYMBOL)
   {
      // Check if negative values are allowed
      if (m_allowNegativeValues)
      {
         SetEditBoxValue(-1 * GetEditBoxValue());
         return;
      }
   }
   else if ((nChar >= '0') && (nChar <= '9'))
   {
      // Get edit box text
      CString valueText;
      GetWindowText(valueText);

      // Get selection range
      DWORD selStart = 0;
      DWORD selEnd = 0;
      SendMessage(EM_GETSEL, (WPARAM)&selStart, (LPARAM)&selEnd);

      // When maximum number of symbols was exceeded do nothing
      if (!valueText.IsEmpty())
      {
         // Remove first character if it is a negation symbol
         if (m_isNegative)
         {
            valueText.Delete(0);
         }

         // Check maximum number of symbols
         if ((valueText.GetLength() >= MAX_LENGTH) && (selStart == selEnd))
         {
            return;
         }
      }

      // When negation symbol exist and cursor is in the first position do nothing
      if (m_isNegative && !selStart && !selEnd)
      {
         return;
      }

      CEdit::OnChar(nChar, nRepCnt, nFlags);
      return;
   }
   else if (nChar == VK_BACK)
   {
      CEdit::OnChar(nChar, nRepCnt, nFlags);
      return;
   }

   ::MessageBeep(MB_ICONEXCLAMATION);
}

void CNumEdit::SetValue(
                        double value)
{
   SetEditBoxValue(value);
}

long CNumEdit::GetIntValue() const
{
   return std::lround(GetEditBoxValue());
}

double CNumEdit::GetDoubleValue() const
{
   return GetEditBoxValue();
}

void CNumEdit::IncreaseValue(
                        long delta)
{
   auto value = GetEditBoxValue();
   value += delta;
   if (!m_allowNegativeValues && value < 0)
   {
      value = 0;
   }

   SetEditBoxValue(value);
}

void CNumEdit::DecreaseValue(
                        long delta)
{
   auto value = GetEditBoxValue();
   value -= delta;
   if (!m_allowNegativeValues && value < 0)
   {
      value = 0;
   }

   SetEditBoxValue(value);
}

void  CNumEdit::NegativeAllowed(
                        bool isNegativeAllowed)
{
   m_allowNegativeValues = isNegativeAllowed;
}

double CNumEdit::GetEditBoxValue() const
{
   // Get edit box text
   CString valueText;
   GetWindowText(valueText);

   // Return 0 when text is empty
   if (valueText.IsEmpty())
   {
      return 0;
   }

   // Remove negation sign if exist
   if (m_isNegative)
   {
      valueText.Delete(0);
   }

   // Convert to a number and multiple by -1 if value is negative
   return (m_isNegative ? -1 : 1) * _tstof(valueText);
}

void CNumEdit::SetEditBoxValue(
                        double value)
{
   // Get selection range
   DWORD selStart = 0;
   DWORD selEnd = 0;
   SendMessage(EM_GETSEL, (WPARAM)&selStart, (LPARAM)&selEnd);

   // Get negation state
   m_isNegative = (m_allowNegativeValues && (value < 0));
   value = abs(value);

   // Convert value to string and add negation sign if needed
   CString valueText;
   valueText.Format(_T("%.9g"), value);
   if (m_isNegative)
   {
      valueText = NEGATION_SYMBOL + valueText;
   }

   // Set edit box text
   SetWindowText(valueText);

   m_lastText = valueText;

   // Set selection range
   SendMessage(EM_SETSEL, selStart, selEnd);
}
